<?php
/*
 * Copyright(C) 2004, 2005 op5 AB
 * All rights reserved.
 *
 */
function snmp_get_if_name($info)
{
	if (empty($info['ifIndex']))
		return false;
	$desc = $alias = '';
	if (!empty($info['ifDescr']) && !is_numeric($info['ifDescr']))
		$desc = $info['ifDescr'];
	if (!empty($info['ifAlias']) && !is_numeric($info['ifAlias']))
		$alias = $info['ifAlias'];

	if (empty($desc) && empty($alias))
		return "Interface " . $info['ifIndex'];

	$name = "IF " . $info['ifIndex'];
	if (!empty($desc))
		$name .= ': ' . $desc;
	if (!empty($alias))
		$name .= ' - ' . $alias;

	$name = preg_replace('/[~!$%^&*|\'"<>?,()=]*/', '', $name);
	$len_max = 240;
	if (strlen($name) > $len_max) {
		$orig_name = $name;
		$name = substr($orig_name, 0, $len_max - 10);
		$name .= '...';
		$name .= substr($orig_name, -10, 10);
	}

	return $name;
}

/**
 * @return an array of interfaces if any. an empty array if no interfaces were
 *         found. false if there was another error
 */
function snmp_interface_scan($ip, $community, $version)
{
	global $DEBUG;
	global $snmp_hide_vlan_interfaces;

	$lines = false;
	$val = false;
	$ret = false;

	$ifDescr_oid = '.1.3.6.1.2.1.2.2.1.2';
	$ifAlias_oid = '.1.3.6.1.2.1.31.1.1.1.18';
	$oid_vars = array
		(
		 'ifSpeed' => '.1.3.6.1.2.1.2.2.1.5',
		 'ifAdminStatus' => '.1.3.6.1.2.1.2.2.1.7',
		 'ifOperStatus' => '.1.3.6.1.2.1.2.2.1.8',
		 'ifInOctets' => '.1.3.6.1.2.1.2.2.1.10',
		 'ifOutOctets' => '.1.3.6.1.2.1.2.2.1.16',
		 );

	# timeout is in micro-seconds, so we multiply by
	# 1000000 to get it in seconds
	$timeout = 5 * 1000000;
	$retries = 2;
	$walker = 'snmprealwalk';
	# this makes php strip the var type from the snmp value
	snmp_set_quick_print(true);
	if($version == 2 || $version === '2c') {
		$version = '2c';
		# SNMPv2 uses 64-bit counters
		$oid_vars['ifInOctets'] = '.1.3.6.1.2.1.31.1.1.1.6';
		$oid_vars['ifOutOctets'] = '.1.3.6.1.2.1.31.1.1.1.10';
		$oid_vars['ifHighSpeed'] = '.1.3.6.1.2.1.31.1.1.1.15';
		$walker = 'snmp2_real_walk';
	}
	@$desc_list = $walker($ip, $community, $ifDescr_oid, $timeout, $retries);

	if (empty($desc_list) || !is_array($desc_list)) {
		return false;
	}

	$interface_list = array();
	foreach ($desc_list as $oid => $desc) {
		$idx = array_pop(explode('.', $oid));
		$interface_list[$idx]['ifDescr'] = $desc;
	}

	@$alias_list = $walker($ip, $community, $ifAlias_oid, $timeout, $retries);
	if (!empty($alias_list)) foreach ($alias_list as $oid => $alias) {
		$alias = trim(str_replace('"', '', $alias));
		if (empty($alias))
			continue;
		$idx = array_pop(explode('.', $oid));
		if (isset($interface_list[$idx]))
			$interface_list[$idx]['ifAlias'] = $alias;
	}
	if ($DEBUG) { echo "<pre>\n"; print_r($interface_list); echo "</pre>\n"; }

	if(empty($interface_list)) {
		return $interface_list;
	}

	if($DEBUG) {
		echo "<pre>\n"; print_r($interface_list); echo "</pre>\n";
	}

	# join the list of oids to pass to the command line
	foreach($interface_list as $index => $ary) {
		$oids = '';
		foreach($oid_vars as $oid) {
			$oids = "$oids $oid.$index";
		}
		$lines = false;
		$cmd = "snmpget -m: -v $version -c $community $ip -Ovq $oids";
		exec($cmd, $lines, $ret);
		if($ret !== 0) {
			# This can happen when a logical interface was queried.
			# The error message will (in that case) be
			# Error in packet
			# Reason (noSuchName) There is no such variable name in this MIB.
			# which means we can be dead sure it's not a real interface, since
			# those, without exceptions, *MUST* have all the queried oid's.
			if($DEBUG) {
				echo "<pre>\nsnmpget returned $ret. $ary[0] is most likely a logical interface<br />\n";
				print_r($lines);
				echo "</pre>\n";
			}
			unset($interface_list[$index]);
			continue;
		}
		$i = 0;
		foreach ($oid_vars as $what => $oid) {
			$interface_list[$index][$what] = $lines[$i++];
		}
	}
	if ($DEBUG) { echo "<pre>\n"; print_r($interface_list); echo "</pre>\n"; }

	# final filtering. We remove all interfaces that aren't active and
	# that have a registered speed of less than 10000 bits per second
	foreach($interface_list as $index => $foo) {
		if($foo['ifSpeed'] <= 10000 ||
		   $foo['ifAdminStatus'] !== '1' ||
		   $foo['ifOperStatus'] !== '1')
		{
			continue;
		}
		if(isset($foo['ifHighSpeed']) && ($foo['ifSpeed'] / 1000000) < $foo['ifHighSpeed']){
			$foo['ifSpeed'] = $foo['ifHighSpeed'] . 'mbit';
		} else {
			$foo['ifSpeed'] = $foo['ifSpeed'] / 1000000 . 'mbit';
		}
		$foo['ifIndex'] = $index;
		$ret_list[$index] = $foo;
	}

	return($ret_list);
}

/**
 * @return an array of interfaces if any. an empty array if no interfaces were
 *         found. false if there was another error
 */
function snmp_interface_scan_v3($ip, $v3_secname, $v3_seclevel, $v3_authprot, $v3_authpass, $v3_privprot, $v3_privpass, $version) {
	global $DEBUG;

	$lines = false;
	$val = false;
	$ret = false;

	$ifDescr_oid = '.1.3.6.1.2.1.2.2.1.2';
	$ifAlias_oid = '.1.3.6.1.2.1.31.1.1.1.18';
	$oid_vars = array
		(
		 'ifSpeed' => '.1.3.6.1.2.1.2.2.1.5',
		 'ifAdminStatus' => '.1.3.6.1.2.1.2.2.1.7',
		 'ifOperStatus' => '.1.3.6.1.2.1.2.2.1.8',
		# SNMPv3 uses 64-bit counters
		 'ifInOctets' => '.1.3.6.1.2.1.31.1.1.1.6',
		 'ifOutOctets' => '.1.3.6.1.2.1.31.1.1.1.10',
         'ifHighSpeed' => '.1.3.6.1.2.1.31.1.1.1.15'
		 );

	# timeout is in micro-seconds, so we multiply by
	# 1000000 to get it in seconds
	$timeout = 5 * 1000000;
	$retries = 2;
	$walker = 'snmp3_real_walk';
	# this makes php strip the var type from the snmp value
	snmp_set_quick_print(true);
#	$desc_list = $walker($ip, $v3_secname, $v3_seclevel, $v3_authprot, $v3_authpass, $v3_privprot, $v3_privpass, $ifDescr_oid, $timeout, $retries);

	# Since we seem to have problems with snmp3_real_walk we use commandline snmpwalk
	if($v3_seclevel === "noAuthNoPriv"){
		$snmp_walk = "snmpwalk -v 3 -Oq $ip $ifDescr_oid -u $v3_secname -l $v3_seclevel";
	} elseif ($v3_seclevel === "authNoPriv"){
		$snmp_walk = "snmpwalk -v 3 -Oq $ip $ifDescr_oid -u $v3_secname -a $v3_authprot -l $v3_seclevel  -A $v3_authpass";
	} else {
		$snmp_walk = "snmpwalk -v 3 -Oq $ip $ifDescr_oid -u $v3_secname -a $v3_authprot -l $v3_seclevel  -A $v3_authpass -x $v3_privprot -X $v3_privpass";
	}
	$man_scan = exec($snmp_walk, $exec_arr, $exec_retval);
	$tmp_arr = explode("\n", $man_scan);
	foreach($exec_arr as $row){
		$interface = explode(" ", $row, 2);
		$desc_list[$interface[0]]=$interface[1];
	}

	if (empty($desc_list) || !is_array($desc_list)) {
		return false;
	}

	$interface_list = array();
	foreach ($desc_list as $oid => $desc) {
		$idx = array_pop(explode('.', $oid));
		$interface_list[$idx]['ifDescr'] = $desc;
	}

	foreach($interface_list as $index => $ary) {
		$lower_desc = strtolower($ary['ifDescr']);
		if(empty($lower_desc) ||
		   strpos($lower_desc, 'null') !== false ||
		   strpos($lower_desc, 'vlan') !== false ||
		   strpos($lower_desc, 'loopback') !== false ||
		   strpos($lower_desc, 'tunnel') !== false)
		{
			if ($DEBUG)
				echo "Not scanning logical interface #$index ($lower_desc)<br />\n";
			unset($interface_list[$index]);
		}
	}

	@$alias_list = $walker($ip, $v3_secname, $v3_seclevel, $v3_authprot, $v3_authpass, $v3_privprot, $v3_privpass, $ifAlias_oid, $timeout, $retries);
	if (!empty($alias_list)) foreach ($alias_list as $oid => $alias) {
		$alias = trim(str_replace('"', '', $alias));
		if (empty($alias))
			continue;
		$idx = array_pop(explode('.', $oid));
		if (isset($interface_list[$idx]))
			$interface_list[$idx]['ifAlias'] = $alias;
	}
	if ($DEBUG) { echo "<pre>\n"; print_r($interface_list); echo "</pre>\n"; }

	if(empty($interface_list)) {
		return $interface_list;
	}

	if($DEBUG) {
		echo "<pre>\n"; print_r($interface_list); echo "</pre>\n";
	}

	# join the list of oids to pass to the command line
	foreach($interface_list as $index => $ary) {
		$oids = '';
		foreach($oid_vars as $oid) {
			$oids = "$oids $oid.$index";
		}
		$lines = false;
		$cmd = "snmpget -m: -v $version -c $community $ip -Ovq $oids";
		$params = " -l $v3_seclevel";
		if ($v3_secname != '') {
		    $params = $params . " -u $v3_secname";
		}
		if (strtolower($v3_seclevel) == 'noauthnopriv') {
		    $params = $params . "";
		} elseif (strtolower($v3_seclevel) == 'authnopriv') {
		    $params = $params . " -a $v3_authprot -A $v3_authpass";
		} elseif (strtolower($v3_seclevel) == 'authpriv') {
		    $params = $params . " -a $v3_authprot -A $v3_authpass" .
			" -x $v3_privprot -X $v3_privpass";
		}
		$cmd = "snmpget -m: -v $version $params $ip -Ovq $oids";
		if($DEBUG) {
			echo "<pre>\n"; print_r($cmd); echo "</pre>\n";
		}
		exec($cmd, $lines, $ret);
		if($ret !== 0) {
			# This can happen when a logical interface was queried.
			# The error message will (in that case) be
			# Error in packet
			# Reason (noSuchName) There is no such variable name in this MIB.
			# which means we can be dead sure it's not a real interface, since
			# those, without exceptions, *MUST* have all the queried oid's.
			if($DEBUG) {
				echo "<pre>\nsnmpget returned $ret. $ary[0] is most likely a logical interface<br />\n";
				print_r($lines);
				echo "</pre>\n";
			}
			unset($interface_list[$index]);
			continue;
		}
		$i = 0;
		foreach ($oid_vars as $what => $oid) {
			$interface_list[$index][$what] = $lines[$i++];
		}
	}
	if ($DEBUG) { echo "<pre>\n"; print_r($interface_list); echo "</pre>\n"; }

	# final filtering. We remove all interfaces that aren't active and
	# that have a registered speed of less than 10000 bits per second
	foreach($interface_list as $index => $foo) {
		if($foo['ifSpeed'] <= 10000 ||
		   $foo['ifAdminStatus'] !== '1' ||
		   $foo['ifOperStatus'] !== '1')
		{
			continue;
		}
		if(isset($foo['ifHighSpeed']) && ($foo['ifSpeed'] / 1000000) < $foo['ifHighSpeed']){
			$foo['ifSpeed'] = $foo['ifHighSpeed'] . 'mbit';
		} else {
			$foo['ifSpeed'] = $foo['ifSpeed'] / 1000000 . 'mbit';
		}
		$foo['ifIndex'] = $index;
		$ret_list[$index] = $foo;
	}

	return($ret_list);
}
